@@ -2,6 +2,13 @@
 # coding=utf-8
 # ------------------------------------------------------------------------
 
+try:
+    from hashlib import md5
+except ImportError:
+    import md5
+
+import sys
+
 from django import forms
 from django.core.cache import cache as django_cache
 from django.conf import settings as django_settings
@@ -18,30 +25,36 @@ from django.db.transaction import commit_on_success
 
 import mptt
 
-from feincms import settings
+from feincms import settings, ensure_completely_loaded
 from feincms.admin import editor
+from feincms.admin import item_editor
 from feincms.management.checker import check_database_schema
-from feincms.models import Base
+from feincms.models import Base, create_base_model
 from feincms.utils import get_object, copy_model_instance
 import feincms.admin.filterspecs
 
 
 # ------------------------------------------------------------------------
-class PageManager(models.Manager):
+class ActiveAwareContentManagerMixin(object):
+    """
+    Implement what's necessary to add some kind of "active" state for content
+    objects. The notion of active is defined by a number of filter rules that
+    must all match (AND) for the object to be active.
+
+    A Manager for a content class using the "datepublisher" extension
+    should either adopt this mixin or implement a similar interface.
+    """
 
     # A list of filters which are used to determine whether a page is active or not.
     # Extended for example in the datepublisher extension (date-based publishing and
     # un-publishing of pages)
-    active_filters = [
-        Q(active=True),
-        ]
-
-    # The fields which should be excluded when creating a copy. The mptt fields are
-    # excluded automatically by other mechanisms
-    exclude_from_copy = ['id', 'tree_id', 'lft', 'rght', 'level']
+    active_filters = ()
 
     @classmethod
     def apply_active_filters(cls, queryset):
+        """
+        Return a queryset reflecting the filters defined.
+        """
         for filt in cls.active_filters:
             if callable(filt):
                 queryset = filt(queryset)
@@ -50,9 +63,46 @@ class PageManager(models.Manager):
 
         return queryset
 
+    @classmethod
+    def add_to_active_filters(cls, filter):
+        """
+        Add a new clause to the active filters. A filter may be either
+        a Q object to be applied to the content class or a callable taking
+        a queryset and spitting out a new one.
+        """
+        if not cls.active_filters:
+            cls.active_filters = list()
+        cls.active_filters.append(filter)
+
     def active(self):
+        """
+        Return only currently active objects.
+        """
         return self.apply_active_filters(self)
 
+# ------------------------------------------------------------------------
+def path_to_cache_key(path):
+    from django.utils.encoding import iri_to_uri
+    path = iri_to_uri(path)
+
+    # logic below borrowed from http://richwklein.com/2009/08/04/improving-django-cache-part-ii/
+    # via acdha's django-sugar
+    if len(path) > 250:
+        m = md5()
+        m.update(path)
+        path = m.hexdigest() + '-' + path[:200]
+
+    cache_key = 'PAGE-FOR-URL-%d-%s' % ( django_settings.SITE_ID, path )
+
+    return cache_key
+
+class PageManager(models.Manager, ActiveAwareContentManagerMixin):
+
+    # The fields which should be excluded when creating a copy. The mptt fields are
+    # excluded automatically by other mechanisms
+    # ???: Then why are the mptt fields listed here?
+    exclude_from_copy = ['id', 'tree_id', 'lft', 'rght', 'level', 'redirect_to']
+
     def page_for_path(self, path, raise404=False):
         """
         Return a page for a path.
@@ -82,8 +132,8 @@ class PageManager(models.Manager):
         Return the best match for a path. If the path as given is unavailable,
         continues to search by chopping path components off the end.
 
-        Tries hard to avoid unnecessary database lookups by generating all poss
-        matching url prefixes and choosing the longtest match.
+        Tries hard to avoid unnecessary database lookups by generating all
+        possible matching URL prefixes and choosing the longtest match.
 
         Page.best_match_for_path('/photos/album/2008/09') might return the
         page with url '/photos/album'.
@@ -93,16 +143,11 @@ class PageManager(models.Manager):
         path = path.strip('/')
 
         # Cache path -> page resolving.
-        # Note: The only possibility that this returns a stale association
-        # is if a page at an url is removed and/or replaced by another page
-        # at that url. This might happen either by manual intervention (admin
-        # moving the page) or automatically via the datepublisher extension
-        # (a page expiring and another becoming active).
-        # Both cases can handle a short time of "being wrong", so this should
-        # be OK.
+        # We flush the cache entry on page saving, so the cache should always
+        # be up to date.
 
         if settings.FEINCMS_USE_CACHE:
-            ck = 'PAGE-FOR-URL-' + path
+            ck = path_to_cache_key(path)
             page = django_cache.get(ck)
             if page:
                 return page
@@ -179,9 +224,21 @@ class PageManager(models.Manager):
 
         return Page.objects.get(pk=with_page.pk)
 
+PageManager.add_to_active_filters( Q(active=True) )
 
 # MARK: -
 # ------------------------------------------------------------------------
+
+try:
+    # MPTT 0.4
+    from mptt.models import MPTTModel
+    mptt_register = False
+    Base = create_base_model(MPTTModel)
+except ImportError:
+    # MPTT 0.3
+    mptt_register = True
+
+
 class Page(Base):
 
     active = models.BooleanField(_('active'), default=False)
@@ -192,7 +249,7 @@ class Page(Base):
     slug = models.SlugField(_('slug'), max_length=150)
     parent = models.ForeignKey('self', verbose_name=_('Parent'), blank=True, null=True, related_name='children')
     parent.parent_filter = True # Custom list_filter - see admin/filterspecs.py
-    in_navigation = models.BooleanField(_('in navigation'), default=True)
+    in_navigation = models.BooleanField(_('in navigation'), default=False)
     override_url = models.CharField(_('override URL'), max_length=300, blank=True,
         help_text=_('Override the target URL. Be sure to include slashes at the beginning and at the end if it is a local URL. This affects both the navigation and subpages\' URLs.'))
     redirect_to = models.CharField(_('redirect to'), max_length=300, blank=True,
@@ -221,6 +278,9 @@ class Page(Base):
         Check whether this page and all its ancestors are active
         """
 
+        if not self.pk:
+            return False
+
         pages = Page.objects.active().filter(tree_id=self.tree_id, lft__lte=self.lft, rght__gte=self.rght)
         return pages.count() > self.level
     is_active.short_description = _('is active')
@@ -236,6 +296,23 @@ class Page(Base):
         queryset = PageManager.apply_active_filters(self.get_ancestors())
         return queryset.count() >= self.level
 
+    def active_children(self):
+        """
+        Returns a queryset describing all active children of the current page.
+        This is different than page.get_descendants (from mptt) as it will
+        additionally select only child pages that are active.
+        """
+        return Page.objects.active().filter(parent=self)
+
+    def active_children_in_navigation(self):
+        """
+        Returns a queryset describing all active children that also have the
+        in_navigation flag set. This might be used eg. in building navigation
+        menues (only show a disclosure indicator if there actually is something
+        to disclose).
+        """
+        return self.active_children().filter(in_navigation=True)
+
     def short_title(self):
         """
         Title shortened for display.
@@ -268,7 +345,7 @@ class Page(Base):
 
         # Okay, we changed the URL -- remove the old stale entry from the cache
         if settings.FEINCMS_USE_CACHE:
-            ck = 'PAGE-FOR-URL-' + self._original_cached_url.strip('/')
+            ck = path_to_cache_key( self._original_cached_url.strip('/') )
             django_cache.delete(ck)
 
         # If our cached URL changed we need to update all descendants to
@@ -325,6 +402,14 @@ class Page(Base):
         """
         return None
 
+    def last_modified(self, request):
+        """
+        Generate a last modified date for this page.
+        Since a standard page has no way of knowing this, we always return
+        "no date" -- this is overridden by the changedate extension.
+        """
+        return None
+
     def setup_request(self, request):
         """
         Before rendering a page, run all registered request processors. A request
@@ -349,6 +434,9 @@ class Page(Base):
         for fn in self.response_processors:
             fn(self, request, response)
 
+        if response.status_code == 200 and not response.has_header('Content-Length'):
+            response['Content-Length'] = len(response.content)
+
     def require_path_active_request_processor(self, request):
         """
         Checks whether any ancestors are actually inaccessible (ie. not
@@ -403,6 +491,10 @@ class Page(Base):
             etag = page.etag(request)
             return etag
 
+        def lastmodifier(request, page, *args, **kwargs):
+            lm = page.last_modified()
+            return lm
+
         # Unavailable in Django 1.0 -- the current implementation of ETag support
         # requires Django 1.1 unfortunately.
         from django.views.decorators.http import condition
@@ -411,7 +503,7 @@ class Page(Base):
         # the net effect is that we will be getting a DummyResponse from
         # the handler if processing is to continue and a non-DummyResponse
         # (should be a "304 not modified") if the etag matches.
-        rsp = condition(etag_func=etagger)(dummy_response_handler)(request, self)
+        rsp = condition(etag_func=etagger, last_modified_func=lastmodifier)(dummy_response_handler)(request, self)
 
         # If dummy then don't do anything, if a real response, return and
         # thus shortcut the request processing.
@@ -426,10 +518,10 @@ class Page(Base):
         """
         etag = self.etag(request)
         if etag is not None:
-            response['ETag'] = etag
+            response['ETag'] = '"' + etag + '"'
 
     @staticmethod
-    def debug_sql_queries_response_processor(verbose=False):
+    def debug_sql_queries_response_processor(verbose=False, file=sys.stderr):
         if not django_settings.DEBUG:
             return lambda self, request, response: None
 
@@ -444,18 +536,18 @@ class Page(Base):
                 pass
 
             if verbose:
-                print "--------------------------------------------------------------"
+                print >> file, "--------------------------------------------------------------"
             time = 0.0
             i = 0
             for q in connection.queries:
                 i += 1
                 if verbose:
-                    print "%d : [%s]\n%s\n" % ( i, q['time'], print_sql(q['sql']))
+                    print >> file, "%d : [%s]\n%s\n" % ( i, q['time'], print_sql(q['sql']))
                 time += float(q['time'])
 
-            print "--------------------------------------------------------------"
-            print "Total: %d queries, %.3f ms" % (i, time)
-            print "--------------------------------------------------------------"
+            print >> file, "--------------------------------------------------------------"
+            print >> file, "Total: %d queries, %.3f ms" % (i, time)
+            print >> file, "--------------------------------------------------------------"
 
         return processor
 
@@ -473,7 +565,8 @@ class Page(Base):
 
 
 # ------------------------------------------------------------------------
-mptt.register(Page)
+if mptt_register: # MPTT 0.3 legacy support
+    mptt.register(Page)
 
 # Our default request processors
 Page.register_request_processors(Page.require_path_active_request_processor,
@@ -485,7 +578,12 @@ signals.post_syncdb.connect(check_database_schema(Page, __name__), weak=False)
 # MARK: -
 # ------------------------------------------------------------------------
 class PageAdminForm(forms.ModelForm):
+    never_copy_fields = ('title', 'slug', 'parent', 'active', 'override_url',
+        'translation_of')
+
     def __init__(self, *args, **kwargs):
+        ensure_completely_loaded()
+
         if 'initial' in kwargs:
             if 'parent' in kwargs['initial']:
                 # Prefill a few form values from the parent page
@@ -498,8 +596,9 @@ class PageAdminForm(forms.ModelForm):
                             del data[field]
 
                     # These are always excluded from prefilling
-                    for field in ('title', 'slug', 'parent', 'active', 'override_url'):
-                        del data[field]
+                    for field in self.never_copy_fields:
+                        if field in data:
+                            del data[field]
 
                     kwargs['initial'].update(data)
                 except Page.DoesNotExist:
@@ -555,8 +654,8 @@ class PageAdminForm(forms.ModelForm):
         # at least for now.
         active_pages = Page.objects.filter(active=True)
 
-        if 'id' in self.initial:
-            current_id = self.initial['id']
+        if self.instance:
+            current_id = self.instance.id
             active_pages = active_pages.exclude(id=current_id)
 
         if not cleaned_data['active']:
@@ -594,15 +693,9 @@ class PageAdminForm(forms.ModelForm):
 
 
 # ------------------------------------------------------------------------
-if settings.FEINCMS_PAGE_USE_SPLIT_PANE_EDITOR:
-    list_modeladmin = editor.SplitPaneEditor
-else:
-    list_modeladmin = editor.TreeEditor
 
-# MARK: -
-# ------------------------------------------------------------------------
 
-class PageAdmin(editor.ItemEditor, list_modeladmin):
+class PageAdmin(editor.ItemEditor, editor.TreeEditor):
     class Media:
         css = {}
         js = []
@@ -611,30 +704,49 @@ class PageAdmin(editor.ItemEditor, list_modeladmin):
 
     # the fieldsets config here is used for the add_view, it has no effect
     # for the change_view which is completely customized anyway
+    unknown_fields = ['override_url', 'redirect_to']
     fieldsets = [
         (None, {
             'fields': ['active', 'in_navigation', 'template_key', 'title', 'slug',
                 'parent'],
         }),
+        item_editor.FEINCMS_CONTENT_FIELDSET,
         (_('Other options'), {
             'classes': ['collapse',],
-            'fields': ['override_url',],
+            'fields': unknown_fields,
         }),
         ]
+    readonly_fields = []
     list_display = ['short_title', 'is_visible_admin', 'in_navigation_toggle', 'template']
     list_filter = ['active', 'in_navigation', 'template_key', 'parent']
     search_fields = ['title', 'slug']
     prepopulated_fields = { 'slug': ('title',), }
 
     raw_id_fields = ['parent']
-    show_on_top = ['title', 'active', 'parent']
     radio_fields = {'template_key': admin.HORIZONTAL}
 
     def __init__(self, *args, **kwargs):
+        ensure_completely_loaded()
+
         if len(Page._feincms_templates) > 4:
             del(self.radio_fields['template_key'])
 
-        return super(PageAdmin, self).__init__(*args, **kwargs)
+        super(PageAdmin, self).__init__(*args, **kwargs)
+
+        # The use of fieldsets makes only fields explicitly listed in there
+        # actually appear in the admin form. However, extensions should not be
+        # aware that there is a fieldsets structure and even less modify it;
+        # we therefore enumerate all of the model's field and forcibly add them
+        # to the last section in the admin. That way, nobody is left behind.
+        from django.contrib.admin.util import flatten_fieldsets
+        present_fields = flatten_fieldsets(self.fieldsets)
+
+        for f in self.model._meta.fields:
+            if not f.name.startswith('_') and not f.name in ('id', 'lft', 'rght', 'tree_id', 'level') and \
+                    not f.auto_created and not f.name in present_fields and f.editable:
+                self.unknown_fields.append(f.name)
+                if not f.editable:
+                    self.readonly_fields.append(f.name)
 
     in_navigation_toggle = editor.ajax_editable_boolean('in_navigation', _('in navigation'))
 
@@ -713,11 +825,12 @@ class PageAdmin(editor.ItemEditor, list_modeladmin):
         return super(PageAdmin, self).change_view(request, object_id, extra_context)
 
     def render_item_editor(self, request, object, context):
-        try:
-            active = Page.objects.active().exclude(pk=object.pk).get(_cached_url=object._cached_url)
-            context['to_replace'] = active
-        except Page.DoesNotExist:
-            pass
+        if object:
+            try:
+                active = Page.objects.active().exclude(pk=object.pk).get(_cached_url=object._cached_url)
+                context['to_replace'] = active
+            except Page.DoesNotExist:
+                pass
 
         return super(PageAdmin, self).render_item_editor(request, object, context)
 
