@@ -264,12 +264,31 @@ def is_parent_of(page1, page2):
     {% if page|is_parent_of:feincms_page %} ... {% endif %}
     """
 
-    return page1.tree_id == page2.tree_id and page1.lft < page2.lft and page1.rght > page2.rght
+    try:
+        return page1.tree_id == page2.tree_id and page1.lft < page2.lft and page1.rght > page2.rght
+    except AttributeError:
+        return False
 
 # ------------------------------------------------------------------------
 @register.filter
 def is_equal_or_parent_of(page1, page2):
-    return page1.tree_id == page2.tree_id and page1.lft <= page2.lft and page1.rght >= page2.rght
+    try:
+        return page1.tree_id == page2.tree_id and page1.lft <= page2.lft and page1.rght >= page2.rght
+    except AttributeError:
+        return False
 
 # ------------------------------------------------------------------------
+@register.filter
+def is_sibling_of(page1, page2):
+    """
+    Determines whether a given page is a sibling of another page
+
+    {% if page|is_sibling_of:feincms_page %} ... {% endif %}
+    """
+
+    try:
+        return page1.parent_id == page2.parent_id
+    except AttributeError:
+        return False
+
 # ------------------------------------------------------------------------
